"""IDs.

Entity-related generators:

.. autosummary::
   :toctree: .

   gene
   protein
   species
   cell_marker
   disease
   tissue
   cell_type
   cell_line
   pathway
   phenotype
   readout
"""

from ._id import (  # noqa
    cell_line,
    cell_marker,
    cell_type,
    disease,
    gene,
    pathway,
    phenotype,
    protein,
    readout,
    species,
    tissue,
)



import os
import tempfile
from unittest.mock import patch

import pytest

from bionty import update_defaults


@pytest.fixture(scope="function")
def current_yaml_replica():
    input_file_content = """
    Species:
        ensembl: release-108
    Gene:
        ensembl: release-108
    """
    with tempfile.NamedTemporaryFile(mode="w+", delete=False) as f:
        f.write(input_file_content)
        f.flush()
        yield f.name

    os.unlink(f.name)


def test_update_defaults(current_yaml_replica):
    with patch("bionty._sync_versions._CURRENT_PATH", current_yaml_replica):
        new_defaults = [
            ("Species", "new_database", "new_version"),
            ("Gene", "new_database", "new_version"),
        ]

        update_defaults(new_defaults, "current")

        with open(current_yaml_replica, "r") as f:
            content = f.read()
            assert (
                "new_database: new_version"
                in content.split("Species:\n", 1)[1].split("\nGene:", 1)[0]
            )



"""v0.4.0.

Revision ID: 267d12e6f6f1
Revises: 7f9a3b24a42b
Create Date: 2022-09-22 23:01:53.991160

"""
import sqlalchemy as sa  # noqa
import sqlmodel  # noqa
from alembic import op

# revision identifiers, used by Alembic.
revision = "267d12e6f6f1"
down_revision = "7f9a3b24a42b"
branch_labels = None
depends_on = None


def upgrade() -> None:
    with op.batch_alter_table("gene", schema=None) as batch_op:
        batch_op.create_index(batch_op.f("ix_gene_gene_type"), ["gene_type"], unique=False)
        batch_op.create_index(batch_op.f("ix_gene_hgnc_id"), ["hgnc_id"], unique=False)
        batch_op.create_index(batch_op.f("ix_gene_mgi_id"), ["mgi_id"], unique=False)
        batch_op.create_index(batch_op.f("ix_gene_omim_id"), ["omim_id"], unique=False)
        batch_op.create_index(batch_op.f("ix_gene_species_id"), ["species_id"], unique=False)
        batch_op.create_index(batch_op.f("ix_gene_synonyms"), ["synonyms"], unique=False)

    with op.batch_alter_table("protein", schema=None) as batch_op:
        batch_op.create_index(
            batch_op.f("ix_protein_ensembl_transcript_ids"),
            ["ensembl_transcript_ids"],
            unique=False,
        )
        batch_op.create_index(batch_op.f("ix_protein_ncbi_gene_ids"), ["ncbi_gene_ids"], unique=False)
        batch_op.create_index(batch_op.f("ix_protein_protein_names"), ["protein_names"], unique=False)


def downgrade() -> None:
    pass



from typing import Literal, Optional

from .._entity import Bionty
from ._shared_docstrings import _doc_params, doc_entites


@_doc_params(doc_entities=doc_entites)
class Pathway(Bionty):
    """Pathway.

    1. Gene Ontology
    Edits of terms are coordinated and reviewed on:
    https://bioportal.bioontology.org/ontologies/GO/?p=summary

    2. Pathway Ontology
    Edits of terms are coordinated and reviewed on:
    https://bioportal.bioontology.org/ontologies/PW/?p=summary

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        species: str = "all",
        source: Optional[Literal["go", "pw"]] = None,
        version: Optional[str] = None,
        **kwargs
    ) -> None:
        super().__init__(source=source, version=version, species=species, **kwargs)



from typing import Literal, Optional, Union

import pandas as pd

from .._entity import Bionty, BiontyField
from .._normalize import GENE_COLUMNS, NormalizeColumns
from ..dev._io import s3_bionty_assets
from ._shared_docstrings import _doc_params, doc_curate, doc_entites


@_doc_params(doc_entities=doc_entites)
class Gene(Bionty):
    """Gene.

    1. Ensembl
    Edits of terms are coordinated and reviewed on:
    https://www.ensembl.org/

    The default indexer is `ensembl_gene_id`

    Args:
        {doc_entities}

    Notes:
        Biotypes: https://www.ensembl.org/info/genome/genebuild/biotypes.html
        Gene Naming: https://www.ensembl.org/info/genome/genebuild/gene_names.html
    """

    def __init__(
        self,
        species: str = "human",
        source: Optional[Literal["ensembl"]] = None,
        version: Optional[str] = None,
        **kwargs,
    ):
        super().__init__(
            source=source,
            version=version,
            species=species,
            reference_id="ensembl_gene_id",
            **kwargs,
        )
        self._lookup_field = "symbol"
        self._synonyms_dict = {"symbol": "synonyms"}

    def df(self) -> pd.DataFrame:
        """DataFrame.

        See ingestion: https://lamin.ai/docs/bionty-assets/ingest/ensembl-gene
        """
        self._filepath = s3_bionty_assets(self._parquet_filename)

        df = pd.read_parquet(self._filepath)
        NormalizeColumns.gene(df, species=self.species)
        try:
            # for pandas > 2.0
            if not pd.api.types.is_any_real_numeric_dtype(df.index):
                df = df.reset_index().copy()
        except AttributeError:
            if not df.index.is_numeric():
                df = df.reset_index().copy()
        df = df[~df[self.reference_id].isnull()]

        return df

    @_doc_params(doc_curate=doc_curate)
    def curate(  # type: ignore
        self,
        df: pd.DataFrame,
        column: str = None,
        reference_id: Union[BiontyField, str] = "ensembl_gene_id",
    ) -> pd.DataFrame:
        """Curate index of passed DataFrame to conform with default identifier.

        In addition to the .curate() in base class, this also performs alias mapping.

        Args:
            {doc_curate}

        Returns:
            The input DataFrame with the curated index and a boolean `__curated__`
            column that indicates compliance with the default identifier.
        """
        reference_id = str(reference_id)
        agg_col = self._synonyms_dict.get(reference_id)
        df = df.copy()

        # if the query column name does not match any columns in the self.df()
        # Bionty assume the query column and the self.reference_id field use
        # the same type of identifier
        orig_column = column
        if column is not None and column not in self.df().columns:
            # normalize the target column
            column_norm = GENE_COLUMNS.get(column)
            if column_norm in df.columns:
                raise ValueError(f"{column_norm} column already exist!")
            else:
                column = reference_id if column_norm is None else column_norm
                df.rename(columns={orig_column: column}, inplace=True)
            agg_col = self._synonyms_dict.get(column)

        return (
            super()
            ._curate(
                df=df,
                column=column,
                agg_col=agg_col,
                reference_id=reference_id,
            )
            .rename(columns={column: orig_column})
        )

    def lookup(self, field: str = "symbol"):
        return super().lookup(field=field)



import pytest

from lnschema_bionty.dev._id import species


def test_hash_id():
    assert "vado" == species("NCBI_10090")
    assert "24Dy" == species("NCBI_10091")
    with pytest.raises(ValueError):
        species("test_12893")



from typing import Iterable, Optional, Union

import bionty as bt
from sqlmodel import SQLModel

from ..dev import id


def fields_from_knowledge(
    locals: dict,
    entity: bt.Bionty,
):
    df = entity.df().reset_index()
    bionty_dict = {}
    if len(locals) > 1:
        raise AssertionError("Only 1 kwarg can be passed when populating fields from bionty!")
    elif len(locals) == 1:
        k = next(iter(locals))
        v = locals[k]
        try:
            bionty_dict = df.set_index(k).loc[v].to_dict()
        except KeyError:
            raise ValueError(f"No entry is found in bionty reference table with '{k}={v}'!\n Try passing a species other than {entity.species}!")
        bionty_dict[k] = v
        if "ontology_id" in bionty_dict:
            bionty_dict["id"] = bionty_dict["ontology_id"]

    return bionty_dict


def create_species_record(species: Union[str, SQLModel]):
    if isinstance(species, SQLModel):
        species_record = species
    elif isinstance(species, str):
        import lamindb as ln

        from .._core import Species

        species_record = ln.select(Species, name=species).one_or_none()
        if species_record is None:
            species_record = Species.from_bionty(name=species)
    else:
        raise TypeError("species must be a string or a SQLModel record!")

    return species_record


def _add_synonym(synonym: Union[str, Iterable], record: SQLModel):
    """Append new synonym to a synonym field."""
    # nothing happens when passing an empty string
    if isinstance(synonym, str):
        if len(synonym) == 0:
            return
        synonym = [synonym]
    else:
        synonym = list(synonym)

    # because we use | as the separator
    if any(["|" in i for i in synonym]):
        raise AssertionError("A synonym can't contain '|'!")

    # only add if the passed synonym aren't already in synonyms
    synonyms = record.synonyms
    if synonyms is None or len(synonyms) == 0:
        synonyms_set = set()
    else:
        synonyms_set = set(synonyms.split("|"))

    synonyms_set.update(synonym)

    synonyms_updated = "|".join([s for s in synonyms_set])
    if len(synonyms_updated) == 0:
        synonyms_updated = None  # type:ignore
    record.synonyms = synonyms_updated


def knowledge(sqlmodel_class):
    name = sqlmodel_class.__name__
    Entity = getattr(bt, name)

    @classmethod
    def bionty(cls, **init_kwargs):
        """Bionty object of the entity."""
        return Entity(**init_kwargs)

    @classmethod
    def from_bionty(
        cls,
        lookup_result: Optional[tuple] = None,
        species: Union[str, SQLModel, None] = None,
        **kwargs,
    ):
        """Auto-complete additional fields based on bionty reference."""
        if species is not None:
            species = create_species_record(species)

        if isinstance(lookup_result, tuple) and lookup_result is not None:
            return sqlmodel_class(lookup_result=lookup_result, species=species)
        else:
            entity = sqlmodel_class.bionty() if species is None else sqlmodel_class.bionty(species=species.name)
            bionty_dict = fields_from_knowledge(locals=kwargs, entity=entity)
            bionty_dict = _encode_id(bionty_dict)

            if "species_id" in sqlmodel_class.__fields__ and species is not None:
                bionty_dict["species"] = species

            return sqlmodel_class(**bionty_dict)

    def _encode_id(pydantic_attrs: dict):
        if "id" in pydantic_attrs:
            id_encoder = getattr(id, sqlmodel_class.bionty()._entity)
            pydantic_attrs["id"] = id_encoder(pydantic_attrs["id"])
        return pydantic_attrs

    orig_init = sqlmodel_class.__init__

    def __init__(self, lookup_result: Optional[tuple] = None, **kwargs):
        if isinstance(lookup_result, tuple) and lookup_result is not None:
            lookup_dict = lookup_result._asdict()  # type:ignore
            lookup_dict = _encode_id(lookup_dict)
            if "species_id" in sqlmodel_class.__fields__:
                if "species" not in kwargs and "species_id" not in kwargs:
                    raise AssertionError("Must specify a species!")
                elif kwargs["species"] is not None:
                    lookup_dict["species"] = create_species_record(kwargs["species"])
            kwargs = lookup_dict

        orig_init(self, **kwargs)

    def add_synonym(self, synonym: Union[str, Iterable]):
        _add_synonym(synonym=synonym, record=self)

    sqlmodel_class.__init__ = __init__
    sqlmodel_class.from_bionty = from_bionty
    sqlmodel_class.bionty = bionty
    sqlmodel_class.add_synonym = add_synonym

    return sqlmodel_class



"""v0.6.3.

Revision ID: 078d771bc2a0
Revises: 4133eb176df0
Create Date: 2023-01-05 15:20:44.311685
"""
import sqlalchemy as sa  # noqa
import sqlmodel  # noqa
from alembic import op

# revision identifiers, used by Alembic.
revision = "078d771bc2a0"
down_revision = "4133eb176df0"
branch_labels = None
depends_on = None


def upgrade() -> None:
    bind = op.get_bind()
    sqlite = bind.engine.name == "sqlite"

    if sqlite:
        prefix, schema = "bionty.", None
    else:
        prefix, schema = "", "bionty"

    if not sqlite:
        # move to new schema
        op.execute(f"ALTER TABLE biontyversions set SCHEMA {schema}")
        op.execute(f"ALTER TABLE currentbiontyversions set SCHEMA {schema}")

    op.rename_table("biontyversions", f"{prefix}bionty_versions", schema=schema)
    op.rename_table("currentbiontyversions", f"{prefix}current_bionty_versions", schema=schema)

    op.drop_index("ix_biontyversions_created_at", table_name=f"{prefix}bionty_versions", schema=schema)
    op.drop_index("ix_biontyversions_created_by", table_name=f"{prefix}bionty_versions", schema=schema)
    op.drop_index(
        "ix_biontyversions_database",
        table_name=f"{prefix}bionty_versions",
        schema=schema,
    )
    op.drop_index(
        "ix_biontyversions_database_v",
        table_name=f"{prefix}bionty_versions",
        schema=schema,
    )
    op.drop_index("ix_biontyversions_entity", table_name=f"{prefix}bionty_versions", schema=schema)
    op.drop_index(
        "ix_biontyversions_updated_at",
        table_name=f"{prefix}bionty_versions",
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_bionty_versions_created_at"),
        f"{prefix}bionty_versions",
        ["created_at"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_bionty_versions_created_by"),
        f"{prefix}bionty_versions",
        ["created_by"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_bionty_versions_database"),
        f"{prefix}bionty_versions",
        ["database"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_bionty_versions_database_v"),
        f"{prefix}bionty_versions",
        ["database_v"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_bionty_versions_entity"),
        f"{prefix}bionty_versions",
        ["entity"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_bionty_versions_updated_at"),
        f"{prefix}bionty_versions",
        ["updated_at"],
        unique=False,
        schema=schema,
    )
    op.drop_index(
        "ix_currentbiontyversions_created_at",
        table_name=f"{prefix}current_bionty_versions",
        schema=schema,
    )
    op.drop_index(
        "ix_currentbiontyversions_created_by",
        table_name=f"{prefix}current_bionty_versions",
        schema=schema,
    )
    op.drop_index(
        "ix_currentbiontyversions_entity",
        table_name=f"{prefix}current_bionty_versions",
        schema=schema,
    )
    op.drop_index(
        "ix_currentbiontyversions_updated_at",
        table_name=f"{prefix}current_bionty_versions",
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_current_bionty_versions_created_at"),
        f"{prefix}current_bionty_versions",
        ["created_at"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_current_bionty_versions_created_by"),
        f"{prefix}current_bionty_versions",
        ["created_by"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_current_bionty_versions_entity"),
        f"{prefix}current_bionty_versions",
        ["entity"],
        unique=True,
        schema=schema,
    )
    op.create_index(
        op.f("ix_bionty_current_bionty_versions_updated_at"),
        f"{prefix}current_bionty_versions",
        ["updated_at"],
        unique=False,
        schema=schema,
    )

    if sqlite:
        with op.batch_alter_table("bionty.bionty_versions", schema=None) as batch_op:
            batch_op.drop_index("ix_bionty_bionty_versions_created_at")
            batch_op.drop_index("ix_bionty_bionty_versions_created_by")
            batch_op.drop_index("ix_bionty_bionty_versions_database")
            batch_op.drop_index("ix_bionty_bionty_versions_database_v")
            batch_op.drop_index("ix_bionty_bionty_versions_entity")
            batch_op.drop_index("ix_bionty_bionty_versions_updated_at")
            batch_op.create_index(batch_op.f("ix_bionty.bionty_versions_created_at"), ["created_at"], unique=False)
            batch_op.create_index(batch_op.f("ix_bionty.bionty_versions_created_by"), ["created_by"], unique=False)
            batch_op.create_index(batch_op.f("ix_bionty.bionty_versions_database"), ["database"], unique=False)
            batch_op.create_index(batch_op.f("ix_bionty.bionty_versions_database_v"), ["database_v"], unique=False)
            batch_op.create_index(batch_op.f("ix_bionty.bionty_versions_entity"), ["entity"], unique=False)
            batch_op.create_index(batch_op.f("ix_bionty.bionty_versions_updated_at"), ["updated_at"], unique=False)

        with op.batch_alter_table("bionty.current_bionty_versions", schema=None) as batch_op:
            batch_op.drop_index("ix_bionty_current_bionty_versions_created_at")
            batch_op.drop_index("ix_bionty_current_bionty_versions_created_by")
            batch_op.drop_index("ix_bionty_current_bionty_versions_entity")
            batch_op.drop_index("ix_bionty_current_bionty_versions_updated_at")
            batch_op.create_index(batch_op.f("ix_bionty.current_bionty_versions_created_at"), ["created_at"], unique=False)
            batch_op.create_index(batch_op.f("ix_bionty.current_bionty_versions_created_by"), ["created_by"], unique=False)
            batch_op.create_index(batch_op.f("ix_bionty.current_bionty_versions_entity"), ["entity"], unique=True)
            batch_op.create_index(batch_op.f("ix_bionty.current_bionty_versions_updated_at"), ["updated_at"], unique=False)


def downgrade() -> None:
    pass



from typing import Literal, Optional

from .._entity import Bionty
from ._shared_docstrings import _doc_params, doc_entites


@_doc_params(doc_entities=doc_entites)
class CellType(Bionty):
    """Cell type ontologies.

    1. Cell ontology
    Edits of terms are coordinated and reviewed on:
    https://github.com/obophenotype/cell-ontology

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        species: str = "human",
        source: Optional[Literal["cl"]] = None,
        version: Optional[str] = None,
        **kwargs
    ) -> None:
        super().__init__(
            source=source,
            version=version,
            species=species,
            include_id_prefixes={"cl": ["CL"]},
            **kwargs
        )



"""v0.18.0."""
import sqlalchemy as sa  # noqa
import sqlmodel  # noqa
from alembic import op

revision = "c3f38ffe9e04"
down_revision = "017ac3ec86c5"


def upgrade() -> None:
    bind = op.get_bind()
    if bind.engine.name == "sqlite":
        op.rename_table(old_table_name="bionty.species", new_table_name="lnschema_bionty_species")
        op.rename_table(old_table_name="bionty.gene", new_table_name="lnschema_bionty_gene")
        op.rename_table(old_table_name="bionty.protein", new_table_name="lnschema_bionty_protein")
        op.rename_table(
            old_table_name="bionty.cell_marker",
            new_table_name="lnschema_bionty_cellmarker",
        )
        op.rename_table(old_table_name="bionty.tissue", new_table_name="lnschema_bionty_tissue")
        op.rename_table(old_table_name="bionty.disease", new_table_name="lnschema_bionty_disease")
        op.rename_table(old_table_name="bionty.cell_type", new_table_name="lnschema_bionty_celltype")
        op.rename_table(old_table_name="bionty.cell_line", new_table_name="lnschema_bionty_cellline")
        op.rename_table(old_table_name="bionty.pathway", new_table_name="lnschema_bionty_pathway")
        op.rename_table(
            old_table_name="bionty.phenotype",
            new_table_name="lnschema_bionty_phenotype",
        )
        op.rename_table(old_table_name="bionty.readout", new_table_name="lnschema_bionty_readout")
        op.rename_table(
            old_table_name="bionty.features_gene",
            new_table_name="lnschema_bionty_featuresgene",
        )
        op.rename_table(
            old_table_name="bionty.features_protein",
            new_table_name="lnschema_bionty_featuresprotein",
        )
        op.rename_table(
            old_table_name="bionty.features_cell_marker",
            new_table_name="lnschema_bionty_featurescellmarker",
        )
        op.rename_table(
            old_table_name="bionty.file_readout",
            new_table_name="lnschema_bionty_filereadout",
        )
        op.rename_table(
            old_table_name="bionty.bionty_versions",
            new_table_name="lnschema_bionty_biontyversions",
        )
        op.rename_table(
            old_table_name="bionty.current_bionty_versions",
            new_table_name="lnschema_bionty_currentbiontyversions",
        )
    else:
        op.rename_table(
            old_table_name="species",
            new_table_name="lnschema_bionty_species",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="gene",
            new_table_name="lnschema_bionty_gene",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="protein",
            new_table_name="lnschema_bionty_protein",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="cell_marker",
            new_table_name="lnschema_bionty_cellmarker",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="tissue",
            new_table_name="lnschema_bionty_tissue",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="disease",
            new_table_name="lnschema_bionty_disease",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="cell_type",
            new_table_name="lnschema_bionty_celltype",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="cell_line",
            new_table_name="lnschema_bionty_cellline",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="pathway",
            new_table_name="lnschema_bionty_pathway",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="phenotype",
            new_table_name="lnschema_bionty_phenotype",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="readout",
            new_table_name="lnschema_bionty_readout",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="features_gene",
            new_table_name="lnschema_bionty_featuresgene",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="features_protein",
            new_table_name="lnschema_bionty_featuresprotein",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="features_cell_marker",
            new_table_name="lnschema_bionty_featurescellmarker",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="file_readout",
            new_table_name="lnschema_bionty_filereadout",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="bionty_versions",
            new_table_name="lnschema_bionty_biontyversions",
            schema="bionty",
        )
        op.rename_table(
            old_table_name="current_bionty_versions",
            new_table_name="lnschema_bionty_currentbiontyversions",
            schema="bionty",
        )
        # there seems to be a bug in alembic autogenerate that doesn't pick this up
        op.execute("alter table bionty.lnschema_bionty_species set schema public")
        op.execute("alter table bionty.lnschema_bionty_gene set schema public")
        op.execute("alter table bionty.lnschema_bionty_protein set schema public")
        op.execute("alter table bionty.lnschema_bionty_cellmarker set schema public")
        op.execute("alter table bionty.lnschema_bionty_tissue set schema public")
        op.execute("alter table bionty.lnschema_bionty_disease set schema public")
        op.execute("alter table bionty.lnschema_bionty_celltype set schema public")
        op.execute("alter table bionty.lnschema_bionty_cellline set schema public")
        op.execute("alter table bionty.lnschema_bionty_pathway set schema public")
        op.execute("alter table bionty.lnschema_bionty_phenotype set schema public")
        op.execute("alter table bionty.lnschema_bionty_readout set schema public")
        op.execute("alter table bionty.lnschema_bionty_featuresgene set schema public")
        op.execute("alter table bionty.lnschema_bionty_featuresprotein set schema public")
        op.execute("alter table bionty.lnschema_bionty_featurescellmarker set schema public")
        op.execute("alter table bionty.lnschema_bionty_filereadout set schema public")
        op.execute("alter table bionty.lnschema_bionty_biontyversions set schema public")
        op.execute("alter table bionty.lnschema_bionty_currentbiontyversions set schema public")


def downgrade() -> None:
    pass



from typing import Literal, Optional, Union

import pandas as pd

from .._entity import Bionty, BiontyField
from ..dev._io import s3_bionty_assets
from ._shared_docstrings import _doc_params, doc_curate, doc_entites


@_doc_params(doc_entities=doc_entites)
class CellMarker(Bionty):
    """Cell markers.

    1. Cell Marker Ontology
    Edits of terms are coordinated and reviewed on:
    http://bio-bigdata.hrbmu.edu.cn/CellMarker/

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        species: str = "human",
        source: Optional[Literal["cellmarker"]] = None,
        version: Optional[str] = None,
        **kwargs
    ) -> None:
        super().__init__(
            source=source,
            version=version,
            species=species,
            reference_id="name",
            **kwargs
        )
        self._synonyms_dict = {"name": "synonyms"}

    def df(self) -> pd.DataFrame:
        """DataFrame.

        See ingestion: https://lamin.ai/docs/bionty-assets/ingest/cell-marker-2.0
        """
        localpath = s3_bionty_assets(self._parquet_filename)
        df = pd.read_parquet(localpath)

        return df

    @_doc_params(doc_curate=doc_curate)
    def curate(  # type: ignore
        self,
        df: pd.DataFrame,
        column: str = None,
        reference_id: Union[BiontyField, str] = "name",
    ) -> pd.DataFrame:
        """Curate index of passed DataFrame to conform with default identifier.

        In addition to the .curate() in base class, this also performs alias mapping.

        Args:
            {doc_curate}

        Returns:
            The input DataFrame with the curated index and a boolean `__curated__`
            column that indicates compliance with the default identifier.
        """
        reference_id = str(reference_id)
        agg_col = self._synonyms_dict.get(reference_id)
        df = df.copy()

        # if the query column name does not match any columns in the self.df()
        # Bionty assume the query column and the self._id_field uses the same type of
        # identifier
        orig_column = column
        if column is not None and column not in self.df().columns:
            # normalize the identifier column
            if column in df.columns:
                raise ValueError("{column_norm} column already exist!")
            else:
                column = reference_id if column is None else column
                df.rename(columns={orig_column: column}, inplace=True)
            agg_col = self._synonyms_dict.get(column)

        return (
            super()
            ._curate(
                df=df,
                column=column,
                agg_col=agg_col,
                reference_id=reference_id,
            )
            .rename(columns={column: orig_column})
        )



from datetime import datetime as datetime
from typing import Optional

import sqlalchemy as sa
from lnschema_core import Features, File, User
from lnschema_core._timestamps import CreatedAt, UpdatedAt
from lnschema_core._users import CreatedBy
from lnschema_core.dev.sqlmodel import get_orm, schema_sqlmodel
from sqlalchemy.orm import relationship
from sqlmodel import Field, Relationship

from . import __name__ as module_name
from . import _name as schema_name
from ._link import (
    FeaturesCellMarker,
    FeaturesGene,
    FeaturesPathway,
    FeaturesProtein,
    FileReadout,
    PathwayGene,
)
from .dev import id as idg
from .dev._bionty import knowledge

# this is for backward compat
schema_sqlmodel(schema_name)

# this is the current way
SQLModel = get_orm(module_name)


@knowledge
class Species(SQLModel, table=True):  # type: ignore
    """Species."""

    id: str = Field(default_factory=idg.species, primary_key=True)
    name: Optional[str] = Field(default=None, index=True, unique=True)
    taxon_id: Optional[int] = Field(default=None, index=True, unique=True)
    scientific_name: Optional[str] = Field(default=None, index=True, unique=True)
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


@knowledge
class Gene(SQLModel, table=True):  # type: ignore
    """Genes."""

    id: str = Field(default_factory=idg.gene, primary_key=True)
    ensembl_gene_id: Optional[str] = Field(default=None, index=True)
    symbol: Optional[str] = Field(default=None, index=True)
    gene_type: Optional[str] = Field(default=None, index=True)
    description: Optional[str] = None
    ncbi_gene_id: Optional[int] = Field(default=None, index=True)
    hgnc_id: Optional[str] = Field(default=None, index=True)
    mgi_id: Optional[str] = Field(default=None, index=True)
    omim_id: Optional[int] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    species_id: Optional[str] = Field(default=None, foreign_key="lnschema_bionty_species.id", index=True)
    species: Species = Relationship()
    version: Optional[str] = None
    features: Features = Relationship(
        back_populates="genes",
        sa_relationship_kwargs=dict(secondary=FeaturesGene.__table__),
    )
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


Features.genes = relationship(Gene, back_populates="features", secondary=FeaturesGene.__table__)


@knowledge
class Protein(SQLModel, table=True):  # type: ignore
    """Proteins."""

    id: str = Field(default_factory=idg.protein, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    uniprotkb_id: Optional[str] = Field(default=None, index=True)
    uniprotkb_name: Optional[str] = Field(default=None, index=True)
    protein_names: Optional[str] = Field(default=None, index=True)
    length: Optional[int] = None
    species_id: Optional[str] = Field(default=None, foreign_key="lnschema_bionty_species.id")
    species: Species = Relationship()
    gene_symbols: Optional[str] = None
    gene_synonyms: Optional[str] = None
    ensembl_transcript_ids: Optional[str] = Field(default=None, index=True)
    ncbi_gene_ids: Optional[str] = Field(default=None, index=True)
    features: Features = Relationship(
        back_populates="proteins",
        sa_relationship_kwargs=dict(secondary=FeaturesProtein.__table__),
    )
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


Features.proteins = relationship(Protein, back_populates="features", secondary=FeaturesProtein.__table__)


@knowledge
class CellMarker(SQLModel, table=True):  # type: ignore
    """Cell markers."""

    id: str = Field(default_factory=idg.cell_marker, primary_key=True)
    name: Optional[str] = Field(default=None, index=True, unique=True)
    ncbi_gene_id: Optional[str] = Field(default=None, index=True)
    gene_symbol: Optional[str] = Field(default=None, index=True)
    gene_name: Optional[str] = Field(default=None, index=True)
    uniprotkb_id: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    species_id: Optional[str] = Field(default=None, foreign_key="lnschema_bionty_species.id")
    species: Species = Relationship()
    features: Features = Relationship(
        back_populates="cell_markers",
        sa_relationship_kwargs=dict(secondary=FeaturesCellMarker.__table__),
    )
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


Features.cell_markers = relationship(CellMarker, back_populates="features", secondary=FeaturesCellMarker.__table__)


@knowledge
class Tissue(SQLModel, table=True):  # type: ignore
    """Tissues."""

    __table_args__ = (sa.UniqueConstraint("name", "ontology_id", name="uq_tissue_name_ontology_id"),)

    id: str = Field(default_factory=idg.tissue, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    short_name: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    ontology_id: Optional[str] = Field(default=None, index=True)
    definition: Optional[str] = Field(default=None, index=True)
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


@knowledge
class CellType(SQLModel, table=True):  # type: ignore
    """Cell types."""

    __table_args__ = (sa.UniqueConstraint("name", "ontology_id", name="uq_celltype_name_ontology_id"),)

    id: str = Field(default_factory=idg.cell_type, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    short_name: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    ontology_id: Optional[str] = Field(default=None, index=True)
    definition: Optional[str] = Field(default=None, index=True)
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


@knowledge
class Disease(SQLModel, table=True):  # type: ignore
    """Diseases."""

    __table_args__ = (sa.UniqueConstraint("name", "ontology_id", name="uq_disease_name_ontology_id"),)

    id: str = Field(default_factory=idg.disease, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    short_name: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    ontology_id: Optional[str] = Field(default=None, index=True)
    definition: Optional[str] = Field(default=None, index=True)
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


@knowledge
class CellLine(SQLModel, table=True):  # type: ignore
    """Cell lines."""

    __table_args__ = (sa.UniqueConstraint("name", "ontology_id", name="uq_cellline_name_ontology_id"),)

    id: str = Field(default_factory=idg.cell_line, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    short_name: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    ontology_id: Optional[str] = Field(default=None, index=True)
    definition: Optional[str] = Field(default=None, index=True)
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


@knowledge
class Pathway(SQLModel, table=True):  # type: ignore
    """Pathways."""

    __table_args__ = (sa.UniqueConstraint("name", "ontology_id", name="uq_pathway_name_ontology_id"),)

    id: str = Field(default_factory=idg.pathway, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    short_name: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    ontology_id: Optional[str] = Field(default=None, index=True)
    definition: Optional[str] = Field(default=None, index=True)
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt

    genes: Gene = Relationship(
        back_populates="pathways",
        sa_relationship_kwargs=dict(secondary=PathwayGene.__table__),
    )


Gene.pathways = relationship(Pathway, back_populates="genes", secondary=PathwayGene.__table__)
Features.pathways = relationship(Pathway, secondary=FeaturesPathway.__table__)
Features.__sqlmodel_relationships__["pathways"] = None


@knowledge
class Phenotype(SQLModel, table=True):  # type: ignore
    """Phenotypes."""

    __table_args__ = (sa.UniqueConstraint("name", "ontology_id", name="uq_phenotype_name_ontology_id"),)

    id: str = Field(default_factory=idg.phenotype, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    short_name: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    ontology_id: Optional[str] = Field(default=None, index=True)
    definition: Optional[str] = Field(default=None, index=True)
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


@knowledge
class Readout(SQLModel, table=True):  # type: ignore
    """Biological readouts."""

    __table_args__ = (sa.UniqueConstraint("name", "ontology_id", name="uq_readout_name_ontology_id"),)

    id: str = Field(default_factory=idg.readout, primary_key=True)
    name: Optional[str] = Field(default=None, index=True)
    short_name: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    ontology_id: Optional[str] = Field(default=None, index=True)
    definition: Optional[str] = Field(default=None, index=True)
    molecule: Optional[str] = None
    instrument: Optional[str] = None
    measurement: Optional[str] = None
    created_by: User = Relationship()
    created_by_id: Optional[str] = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt

    files: File = Relationship(
        back_populates="readouts",
        sa_relationship_kwargs=dict(secondary=FileReadout.__table__),
    )


File.readouts = relationship(
    Readout,
    back_populates="files",
    secondary=FileReadout.__table__,
)
File.__sqlmodel_relationships__["readouts"] = None



from datetime import datetime as datetime
from typing import Optional

from lnschema_core._timestamps import CreatedAt, UpdatedAt
from lnschema_core._users import CreatedBy
from lnschema_core.dev.sqlmodel import get_orm
from sqlmodel import Field

from .. import __name__ as module_name

SQLModel = get_orm(module_name)


class BiontyVersions(SQLModel, table=True):  # type: ignore
    """Versions of the knowledge tables."""

    id: Optional[int] = Field(default=None, primary_key=True)
    entity: str = Field(index=True)
    database: str = Field(index=True)
    database_v: str = Field(index=True)
    database_url: Optional[str] = None
    created_by: str = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt


class CurrentBiontyVersions(SQLModel, table=True):  # type: ignore
    """In-use version of the knowledge tables."""

    id: int = Field(primary_key=True, foreign_key=BiontyVersions.id)
    entity: str = Field(index=True, unique=True)
    created_by: str = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt



"""v0.17.0."""
import lnschema_core
import sqlalchemy as sa
import sqlmodel
from alembic import op
from lnschema_core.dev.sqlmodel import get_sqlite_prefix_schema_delim_from_alembic
from packaging import version

revision = "017ac3ec86c5"
down_revision = "3bd9e094a433"

if version.parse(lnschema_core.__version__) >= version.parse("0.34a2"):
    user_table_name = "lnschema_core_user"
    referent_schema = "public"
else:
    user_table_name = "user"
    referent_schema = "core"


def upgrade() -> None:
    sqlite, prefix, schema, delim = get_sqlite_prefix_schema_delim_from_alembic()

    with op.batch_alter_table(f"{prefix}cell_line", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_line_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_line_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_line_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}cell_line_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}cell_marker", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_marker_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_marker_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_marker_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}cell_marker_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}cell_type", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_type_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_type_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}cell_type_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}cell_type_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}disease", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}disease_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}disease_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}disease_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}disease_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}gene", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(batch_op.f(f"ix_bionty{delim}gene_created_at"), ["created_at"], unique=False)
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}gene_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(batch_op.f(f"ix_bionty{delim}gene_updated_at"), ["updated_at"], unique=False)
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}gene_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}pathway", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}pathway_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}pathway_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}pathway_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}pathway_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}phenotype", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}phenotype_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}phenotype_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}phenotype_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}phenotype_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}protein", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}protein_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}protein_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}protein_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}protein_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}species", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}species_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}species_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}species_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}species_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    with op.batch_alter_table(f"{prefix}tissue", schema=schema) as batch_op:
        batch_op.add_column(sa.Column("created_by_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(
            sa.Column(
                "created_at",
                sa.DateTime(),
                server_default=sa.text("(CURRENT_TIMESTAMP)"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("updated_at", sa.DateTime(), nullable=True))
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}tissue_created_at"),
            ["created_at"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}tissue_created_by"),
            ["created_by_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f(f"ix_bionty{delim}tissue_updated_at"),
            ["updated_at"],
            unique=False,
        )
        batch_op.create_foreign_key(
            batch_op.f(f"fk_bionty{delim}tissue_created_by_user"),
            user_table_name,
            ["created_by_id"],
            ["id"],
            referent_schema=referent_schema,
        )

    op.alter_column(
        table_name=f"{prefix}readout",
        column_name="created_by",
        new_column_name="created_by_id",
        schema=schema,
    )


def downgrade() -> None:
    pass



"""v0.7.0.

Revision ID: 6cbe4aa9aaec
Revises: 076bc2188ec3
Create Date: 2023-01-05 21:31:43.834874

"""
import sqlalchemy as sa  # noqa
import sqlmodel  # noqa
from alembic import op

# revision identifiers, used by Alembic.
revision = "6cbe4aa9aaec"
down_revision = "076bc2188ec3"
branch_labels = None
depends_on = None


def upgrade() -> None:
    bind = op.get_bind()
    sqlite = bind.engine.name == "sqlite"

    if sqlite:
        prefix, schema = "bionty.", None
    else:
        prefix, schema = "", "bionty"

    op.alter_column(
        f"{prefix}species",
        column_name="common_name",
        new_column_name="name",
        schema=schema,
    )

    if sqlite:
        with op.batch_alter_table(f"{prefix}species") as batch_op:
            batch_op.drop_index("ix_bionty.species_common_name")
            batch_op.create_index(batch_op.f("ix_bionty.species_name"), ["name"], unique=True)
    else:
        op.drop_index("ix_bionty_species_common_name", table_name="species", schema=schema)
        op.create_index(
            op.f("ix_bionty_species_name"),
            "species",
            ["name"],
            unique=True,
            schema=schema,
        )


def downgrade() -> None:
    pass



import pandas as pd

import bionty as bt


def test_hp_phenotype_curation_ontology_id():
    df = pd.DataFrame(
        index=[
            "HP:0001328",
            "HP:0001332",
            "HP:0001342",
            "HP:0001350",
            "This phenotype does not exist",
        ]
    )

    pt = bt.Phenotype(source="hp", version="2023-01-27")
    curated_df = pt.curate(df)

    curation = curated_df["__curated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert curation.equals(expected_series)


def test_hp_phenotype_curation_name():
    df = pd.DataFrame(
        index=[
            "Specific learning disability",
            "Dystonia",
            "Cerebral hemorrhage",
            "Slurred speech",
            "This phenotype does not exist",
        ]
    )

    pt = bt.Phenotype(source="hp", version="2023-01-27")
    curated_df = pt.curate(df, reference_id=pt.name)

    curation = curated_df["__curated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert curation.equals(expected_series)



"""Link tables.

.. autosummary::
   :toctree: .

   FeaturesGene
   FeaturesProtein
   FeaturesCellMarker
   FeaturesPathway
   FileReadout
   PathwayGene
"""

from ._link import (  # noqa
    FeaturesCellMarker,
    FeaturesGene,
    FeaturesPathway,
    FeaturesProtein,
    FileReadout,
    PathwayGene,
)



import os
import sys

import nox
from laminci import move_built_docs_to_docs_slash_project_slug, upload_docs_artifact
from laminci.nox import build_docs, login_testuser1, run_pre_commit, run_pytest

nox.options.default_venv_backend = "none"


@nox.session
def lint(session: nox.Session) -> None:
    run_pre_commit(session)


@nox.session
@nox.parametrize("package", ["bionty", "lnschema-bionty"])
def build(session, package):
    session.run(*"pip install .[dev,test]".split())
    if package == "bionty":
        run_pytest(session)
        build_docs(session)
        upload_docs_artifact(aws=True)
        move_built_docs_to_docs_slash_project_slug()
    else:
        # navigate into submodule so that lamin-project.yml is correctly read
        os.chdir("./lnschema-bionty")
        session.run(*"pip install .[test]".split())
        session.run(*"git clone https://github.com/laminlabs/lamindb --depth 1".split())
        if sys.platform.startswith("linux"):  # remove version pin when running on CI
            session.run(*"sed -i /lnschema_bionty/d ./lamindb/pyproject.toml".split())
        session.run(*"pip install ./lamindb".split())
        login_testuser1(session)
        session.run(
            "lamin",
            "init",
            "--storage",
            "./docs/guide/lnbionty-test",
            "--schema",
            "bionty",
        )
        session.run("pytest", "-s", "./tests", "--ignore", "./tests/test_migrations.py")



import pandas as pd

import bionty as bt


def test_dron_drug_curation_ontology_id():
    df = pd.DataFrame(
        index=[
            "DRON:00018440",
            "DRON:00018432",
            "DRON:00018438",
            "DRON:00018452",
            "This drug does not exist",
        ]
    )
    dt = bt.Drug(source="dron", version="2023-03-10")
    curated_df = dt.curate(df)

    curation = curated_df["__curated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert curation.equals(expected_series)


def test_dron_drug_curation_name():
    df = pd.DataFrame(
        index=[
            "LILIUM LONGIFLORIUM",
            "citrus bioflavonoids",
            "Ornithine, (L)-Isomer",
            "Hyoscyamus extract",
            "This disease does not exist",
        ]
    )

    dt = bt.Drug(source="dron", version="2023-03-10")
    curated_df = dt.curate(df, reference_id=dt.name)

    curation = curated_df["__curated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert curation.equals(expected_series)



"""v0.18.0b."""
import sqlalchemy as sa
import sqlmodel
from alembic import op
from lnschema_core.dev.sqlmodel import get_sqlite_prefix_schema_delim_from_alembic

revision = "c5ebdad85b31"
down_revision = "c3f38ffe9e04"


def upgrade() -> None:
    sqlite, prefix, schema, delim = get_sqlite_prefix_schema_delim_from_alembic()

    op.drop_index(f"ix_bionty{delim}bionty_versions_created_at", table_name="lnschema_bionty_biontyversions")
    op.drop_index(f"ix_bionty{delim}bionty_versions_created_by", table_name="lnschema_bionty_biontyversions")
    op.drop_index(f"ix_bionty{delim}bionty_versions_database", table_name="lnschema_bionty_biontyversions")
    op.drop_index(f"ix_bionty{delim}bionty_versions_database_v", table_name="lnschema_bionty_biontyversions")
    op.drop_index(f"ix_bionty{delim}bionty_versions_entity", table_name="lnschema_bionty_biontyversions")
    op.drop_index(f"ix_bionty{delim}bionty_versions_updated_at", table_name="lnschema_bionty_biontyversions")
    op.create_index(op.f("ix_lnschema_bionty_biontyversions_created_at"), "lnschema_bionty_biontyversions", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_biontyversions_created_by"), "lnschema_bionty_biontyversions", ["created_by"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_biontyversions_database"), "lnschema_bionty_biontyversions", ["database"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_biontyversions_database_v"), "lnschema_bionty_biontyversions", ["database_v"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_biontyversions_entity"), "lnschema_bionty_biontyversions", ["entity"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_biontyversions_updated_at"), "lnschema_bionty_biontyversions", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}cell_line_created_at", table_name="lnschema_bionty_cellline")
    op.drop_index(f"ix_bionty{delim}cell_line_created_by", table_name="lnschema_bionty_cellline")
    op.drop_index(f"ix_bionty{delim}cell_line_name", table_name="lnschema_bionty_cellline")
    op.drop_index(f"ix_bionty{delim}cell_line_ontology_id", table_name="lnschema_bionty_cellline")
    op.drop_index(f"ix_bionty{delim}cell_line_updated_at", table_name="lnschema_bionty_cellline")
    op.create_index(op.f("ix_lnschema_bionty_cellline_created_at"), "lnschema_bionty_cellline", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellline_created_by_id"), "lnschema_bionty_cellline", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellline_ontology_id"), "lnschema_bionty_cellline", ["ontology_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellline_updated_at"), "lnschema_bionty_cellline", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}cell_marker_created_at", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_created_by", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_gene_name", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_gene_symbol", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_name", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_ncbi_gene_id", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_synonyms", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_uniprotkb_id", table_name="lnschema_bionty_cellmarker")
    op.drop_index(f"ix_bionty{delim}cell_marker_updated_at", table_name="lnschema_bionty_cellmarker")
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_created_at"), "lnschema_bionty_cellmarker", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_created_by_id"), "lnschema_bionty_cellmarker", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_gene_name"), "lnschema_bionty_cellmarker", ["gene_name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_gene_symbol"), "lnschema_bionty_cellmarker", ["gene_symbol"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_name"), "lnschema_bionty_cellmarker", ["name"], unique=True)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_ncbi_gene_id"), "lnschema_bionty_cellmarker", ["ncbi_gene_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_synonyms"), "lnschema_bionty_cellmarker", ["synonyms"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_uniprotkb_id"), "lnschema_bionty_cellmarker", ["uniprotkb_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_cellmarker_updated_at"), "lnschema_bionty_cellmarker", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}cell_type_created_at", table_name="lnschema_bionty_celltype")
    op.drop_index(f"ix_bionty{delim}cell_type_created_by", table_name="lnschema_bionty_celltype")
    op.drop_index(f"ix_bionty{delim}cell_type_name", table_name="lnschema_bionty_celltype")
    op.drop_index(f"ix_bionty{delim}cell_type_ontology_id", table_name="lnschema_bionty_celltype")
    op.drop_index(f"ix_bionty{delim}cell_type_updated_at", table_name="lnschema_bionty_celltype")
    op.create_index(op.f("ix_lnschema_bionty_celltype_created_at"), "lnschema_bionty_celltype", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_celltype_created_by_id"), "lnschema_bionty_celltype", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_celltype_name"), "lnschema_bionty_celltype", ["name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_celltype_ontology_id"), "lnschema_bionty_celltype", ["ontology_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_celltype_updated_at"), "lnschema_bionty_celltype", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}current_bionty_versions_created_at", table_name="lnschema_bionty_currentbiontyversions")
    op.drop_index(f"ix_bionty{delim}current_bionty_versions_created_by", table_name="lnschema_bionty_currentbiontyversions")
    op.drop_index(f"ix_bionty{delim}current_bionty_versions_entity", table_name="lnschema_bionty_currentbiontyversions")
    op.drop_index(f"ix_bionty{delim}current_bionty_versions_updated_at", table_name="lnschema_bionty_currentbiontyversions")
    op.create_index(
        op.f("ix_lnschema_bionty_currentbiontyversions_created_at"), "lnschema_bionty_currentbiontyversions", ["created_at"], unique=False
    )
    op.create_index(
        op.f("ix_lnschema_bionty_currentbiontyversions_created_by"), "lnschema_bionty_currentbiontyversions", ["created_by"], unique=False
    )
    op.create_index(op.f("ix_lnschema_bionty_currentbiontyversions_entity"), "lnschema_bionty_currentbiontyversions", ["entity"], unique=True)
    op.create_index(
        op.f("ix_lnschema_bionty_currentbiontyversions_updated_at"), "lnschema_bionty_currentbiontyversions", ["updated_at"], unique=False
    )

    op.drop_index(f"ix_bionty{delim}disease_created_at", table_name="lnschema_bionty_disease")
    op.drop_index(f"ix_bionty{delim}disease_created_by", table_name="lnschema_bionty_disease")
    op.drop_index(f"ix_bionty{delim}disease_name", table_name="lnschema_bionty_disease")
    op.drop_index(f"ix_bionty{delim}disease_ontology_id", table_name="lnschema_bionty_disease")
    op.drop_index(f"ix_bionty{delim}disease_updated_at", table_name="lnschema_bionty_disease")
    op.create_index(op.f("ix_lnschema_bionty_disease_created_at"), "lnschema_bionty_disease", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_disease_created_by_id"), "lnschema_bionty_disease", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_disease_name"), "lnschema_bionty_disease", ["name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_disease_ontology_id"), "lnschema_bionty_disease", ["ontology_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_disease_updated_at"), "lnschema_bionty_disease", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}gene_created_at", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_created_by", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_ensembl_gene_id", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_gene_type", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_hgnc_id", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_mgi_id", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_ncbi_gene_id", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_omim_id", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_species_id", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_symbol", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_synonyms", table_name="lnschema_bionty_gene")
    op.drop_index(f"ix_bionty{delim}gene_updated_at", table_name="lnschema_bionty_gene")
    op.create_index(op.f("ix_lnschema_bionty_gene_created_at"), "lnschema_bionty_gene", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_created_by_id"), "lnschema_bionty_gene", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_ensembl_gene_id"), "lnschema_bionty_gene", ["ensembl_gene_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_gene_type"), "lnschema_bionty_gene", ["gene_type"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_hgnc_id"), "lnschema_bionty_gene", ["hgnc_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_mgi_id"), "lnschema_bionty_gene", ["mgi_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_ncbi_gene_id"), "lnschema_bionty_gene", ["ncbi_gene_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_omim_id"), "lnschema_bionty_gene", ["omim_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_species_id"), "lnschema_bionty_gene", ["species_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_symbol"), "lnschema_bionty_gene", ["symbol"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_synonyms"), "lnschema_bionty_gene", ["synonyms"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_gene_updated_at"), "lnschema_bionty_gene", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}pathway_created_at", table_name="lnschema_bionty_pathway")
    op.drop_index(f"ix_bionty{delim}pathway_created_by", table_name="lnschema_bionty_pathway")
    op.drop_index(f"ix_bionty{delim}pathway_name", table_name="lnschema_bionty_pathway")
    op.drop_index(f"ix_bionty{delim}pathway_ontology_id", table_name="lnschema_bionty_pathway")
    op.drop_index(f"ix_bionty{delim}pathway_updated_at", table_name="lnschema_bionty_pathway")
    op.create_index(op.f("ix_lnschema_bionty_pathway_created_at"), "lnschema_bionty_pathway", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_pathway_created_by_id"), "lnschema_bionty_pathway", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_pathway_name"), "lnschema_bionty_pathway", ["name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_pathway_ontology_id"), "lnschema_bionty_pathway", ["ontology_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_pathway_updated_at"), "lnschema_bionty_pathway", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}phenotype_created_at", table_name="lnschema_bionty_phenotype")
    op.drop_index(f"ix_bionty{delim}phenotype_created_by", table_name="lnschema_bionty_phenotype")
    op.drop_index(f"ix_bionty{delim}phenotype_name", table_name="lnschema_bionty_phenotype")
    op.drop_index(f"ix_bionty{delim}phenotype_ontology_id", table_name="lnschema_bionty_phenotype")
    op.drop_index(f"ix_bionty{delim}phenotype_updated_at", table_name="lnschema_bionty_phenotype")
    op.create_index(op.f("ix_lnschema_bionty_phenotype_created_at"), "lnschema_bionty_phenotype", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_phenotype_created_by_id"), "lnschema_bionty_phenotype", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_phenotype_name"), "lnschema_bionty_phenotype", ["name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_phenotype_ontology_id"), "lnschema_bionty_phenotype", ["ontology_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_phenotype_updated_at"), "lnschema_bionty_phenotype", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}protein_created_at", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_created_by", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_ensembl_transcript_ids", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_name", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_ncbi_gene_ids", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_protein_names", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_uniprotkb_id", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_uniprotkb_name", table_name="lnschema_bionty_protein")
    op.drop_index(f"ix_bionty{delim}protein_updated_at", table_name="lnschema_bionty_protein")
    op.create_index(op.f("ix_lnschema_bionty_protein_created_at"), "lnschema_bionty_protein", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_created_by_id"), "lnschema_bionty_protein", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_ensembl_transcript_ids"), "lnschema_bionty_protein", ["ensembl_transcript_ids"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_name"), "lnschema_bionty_protein", ["name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_ncbi_gene_ids"), "lnschema_bionty_protein", ["ncbi_gene_ids"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_protein_names"), "lnschema_bionty_protein", ["protein_names"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_uniprotkb_id"), "lnschema_bionty_protein", ["uniprotkb_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_uniprotkb_name"), "lnschema_bionty_protein", ["uniprotkb_name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_protein_updated_at"), "lnschema_bionty_protein", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}readout_created_at", table_name="lnschema_bionty_readout")
    op.drop_index(f"ix_bionty{delim}readout_created_by", table_name="lnschema_bionty_readout")
    op.drop_index(f"ix_bionty{delim}readout_efo_id", table_name="lnschema_bionty_readout")
    op.drop_index(f"ix_bionty{delim}readout_updated_at", table_name="lnschema_bionty_readout")
    op.create_index(op.f("ix_lnschema_bionty_readout_created_at"), "lnschema_bionty_readout", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_readout_created_by_id"), "lnschema_bionty_readout", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_readout_updated_at"), "lnschema_bionty_readout", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}species_created_at", table_name="lnschema_bionty_species")
    op.drop_index(f"ix_bionty{delim}species_created_by", table_name="lnschema_bionty_species")
    op.drop_index(f"ix_bionty{delim}species_name", table_name="lnschema_bionty_species")
    op.drop_index(f"ix_bionty{delim}species_scientific_name", table_name="lnschema_bionty_species")
    op.drop_index(f"ix_bionty{delim}species_taxon_id", table_name="lnschema_bionty_species")
    op.drop_index(f"ix_bionty{delim}species_updated_at", table_name="lnschema_bionty_species")
    op.create_index(op.f("ix_lnschema_bionty_species_created_at"), "lnschema_bionty_species", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_species_created_by_id"), "lnschema_bionty_species", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_species_name"), "lnschema_bionty_species", ["name"], unique=True)
    op.create_index(op.f("ix_lnschema_bionty_species_scientific_name"), "lnschema_bionty_species", ["scientific_name"], unique=True)
    op.create_index(op.f("ix_lnschema_bionty_species_taxon_id"), "lnschema_bionty_species", ["taxon_id"], unique=True)
    op.create_index(op.f("ix_lnschema_bionty_species_updated_at"), "lnschema_bionty_species", ["updated_at"], unique=False)

    op.drop_index(f"ix_bionty{delim}tissue_created_at", table_name="lnschema_bionty_tissue")
    op.drop_index(f"ix_bionty{delim}tissue_created_by", table_name="lnschema_bionty_tissue")
    op.drop_index(f"ix_bionty{delim}tissue_name", table_name="lnschema_bionty_tissue")
    op.drop_index(f"ix_bionty{delim}tissue_ontology_id", table_name="lnschema_bionty_tissue")
    op.drop_index(f"ix_bionty{delim}tissue_updated_at", table_name="lnschema_bionty_tissue")
    op.create_index(op.f("ix_lnschema_bionty_tissue_created_at"), "lnschema_bionty_tissue", ["created_at"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_tissue_created_by_id"), "lnschema_bionty_tissue", ["created_by_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_tissue_name"), "lnschema_bionty_tissue", ["name"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_tissue_ontology_id"), "lnschema_bionty_tissue", ["ontology_id"], unique=False)
    op.create_index(op.f("ix_lnschema_bionty_tissue_updated_at"), "lnschema_bionty_tissue", ["updated_at"], unique=False)

    with op.batch_alter_table("lnschema_bionty_cellline", schema=None) as batch_op:
        batch_op.add_column(sa.Column("short_name", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("synonyms", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("definition", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_cellline_definition"),
            ["definition"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_cellline_short_name"),
            ["short_name"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_cellline_synonyms"),
            ["synonyms"],
            unique=False,
        )
        batch_op.create_unique_constraint("uq_cellline_name_ontology_id", ["name", "ontology_id"])

    with op.batch_alter_table("lnschema_bionty_celltype", schema=None) as batch_op:
        batch_op.add_column(sa.Column("short_name", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("synonyms", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("definition", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_celltype_definition"),
            ["definition"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_celltype_short_name"),
            ["short_name"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_celltype_synonyms"),
            ["synonyms"],
            unique=False,
        )
        batch_op.create_unique_constraint("uq_celltype_name_ontology_id", ["name", "ontology_id"])

    with op.batch_alter_table("lnschema_bionty_disease", schema=None) as batch_op:
        batch_op.add_column(sa.Column("short_name", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("synonyms", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("definition", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_disease_definition"),
            ["definition"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_disease_short_name"),
            ["short_name"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_disease_synonyms"),
            ["synonyms"],
            unique=False,
        )
        batch_op.create_unique_constraint("uq_disease_name_ontology_id", ["name", "ontology_id"])

    with op.batch_alter_table("lnschema_bionty_pathway", schema=None) as batch_op:
        batch_op.add_column(sa.Column("short_name", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("synonyms", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("definition", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_pathway_definition"),
            ["definition"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_pathway_short_name"),
            ["short_name"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_pathway_synonyms"),
            ["synonyms"],
            unique=False,
        )
        batch_op.create_unique_constraint("uq_pathway_name_ontology_id", ["name", "ontology_id"])

    with op.batch_alter_table("lnschema_bionty_phenotype", schema=None) as batch_op:
        batch_op.add_column(sa.Column("short_name", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("synonyms", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("definition", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_phenotype_definition"),
            ["definition"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_phenotype_short_name"),
            ["short_name"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_phenotype_synonyms"),
            ["synonyms"],
            unique=False,
        )
        batch_op.create_unique_constraint("uq_phenotype_name_ontology_id", ["name", "ontology_id"])

    with op.batch_alter_table("lnschema_bionty_readout", schema=None) as batch_op:
        batch_op.add_column(sa.Column("short_name", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("synonyms", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("ontology_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("definition", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.create_index(batch_op.f("ix_lnschema_bionty_readout_name"), ["name"], unique=False)
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_readout_ontology_id"),
            ["ontology_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_readout_short_name"),
            ["short_name"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_readout_synonyms"),
            ["synonyms"],
            unique=False,
        )
        batch_op.create_unique_constraint("uq_readout_name_ontology_id", ["name", "ontology_id"])
        batch_op.drop_column("efo_id")

    with op.batch_alter_table("lnschema_bionty_tissue", schema=None) as batch_op:
        batch_op.add_column(sa.Column("short_name", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("synonyms", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.add_column(sa.Column("definition", sqlmodel.sql.sqltypes.AutoString(), nullable=True))
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_tissue_definition"),
            ["definition"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_lnschema_bionty_tissue_short_name"),
            ["short_name"],
            unique=False,
        )
        batch_op.create_index(batch_op.f("ix_lnschema_bionty_tissue_synonyms"), ["synonyms"], unique=False)
        batch_op.create_unique_constraint("uq_tissue_name_ontology_id", ["name", "ontology_id"])


def downgrade() -> None:
    pass



import pytest

import bionty as bt


@pytest.fixture(scope="session")
def ct():
    return bt.CellType(source="cl", version="2023-02-15")


def test_fuzzy_match_name(ct):
    assert ct.fuzzy_match("T cells", ct.name).index == "T cell"


def test_fuzzy_match_synonyms(ct):
    assert ct.fuzzy_match("P cells", ct.name).index == "nodal myocyte"


def test_fuzzy_match_synonyms_field_none(ct):
    assert ct.fuzzy_match("P cell", ct.name, synonyms_field=None).index == "PP cell"


def test_fuzzy_match_return_ranked_results(ct):
    assert ct.fuzzy_match("P cell", ct.name, return_ranked_results=True).shape == (
        2681,
        5,
    )


def test_fuzzy_match_return_tie_results(ct):
    assert ct.fuzzy_match("A cell", ct.name, synonyms_field=None).shape[0] == 2
